@@ -15,8 +15,9 @@ from .models.chat_item import ChatItem, OuterContextItem, InnerContextItem, Repl
 from .models.enums import MTRSLabelEnum, DiagnosticsXMLTagEnum, MTRSXMLTagEnum, DoctorChoiceXMLTagEnum
 from .models.tracks import TrackInfo, DomainInfo
 from .models.widget import Widget
-from .utils import make_session_id
+from .utils import make_session_id, chunked
 from .xml_parser import XMLParser
+from .utils_import import load_main_objects
 
 __all__ = [
     "AIMessage",
@@ -42,6 +43,8 @@ __all__ = [
     "TrackInfo",
     "Widget",
     "XMLParser",
+    "chunked",
+    "load_main_objects",
     "make_content",
     "make_session_id",
 ]
