@@ -6,7 +6,7 @@ from pathlib import Path
 from zipfile import ZipFile, is_zipfile
 
 ResourceId = str
-ASCII_DIGITS = set(string.ascii_lowercase + string.digits)
+ASCII_DIGITS_SPECIAL = set(string.ascii_lowercase + string.digits + "-")
 SUFFIX_DIR = ".dir"
 SUFFIX_METADATA = ".metadata"
 
@@ -18,7 +18,7 @@ def _validate_exist(files_dir):
 
 
 def _validate_dtype(dtype: str):
-    if all(map(ASCII_DIGITS.__contains__, dtype)):
+    if all(map(ASCII_DIGITS_SPECIAL.__contains__, dtype)):
         return
     raise ValueError(f"Bad dtype: {dtype}")
 
@@ -61,6 +61,12 @@ class FileStorage:
 
         return str(fpath)
 
+    def get_metadata(self, resource_id: ResourceId) -> dict | None:
+        metadata_path = Path(resource_id).with_suffix(SUFFIX_METADATA)
+        if not metadata_path.exists():
+            return None
+        return json.loads(metadata_path.read_text())
+
     async def upload_async(self, content: bytes | str, fname: str) -> ResourceId:
         return self.upload(content, fname)
 
