@@ -4,13 +4,13 @@ from copy import deepcopy
 from datetime import datetime
 from typing import Any, Literal, TypeVar
 
-from mmar_mapi.models.chat_item import ChatItem, ReplicaItem, OuterContextItem
-from mmar_mapi.models.widget import Widget
-from mmar_mapi.type_union import TypeUnion
 from pydantic import Field, ValidationError
 
-from .base import Base
+from mmar_mapi.models.chat_item import ChatItem, OuterContextItem, ReplicaItem
+from mmar_mapi.models.widget import Widget
+from mmar_mapi.type_union import TypeUnion
 
+from .base import Base
 
 _DT_FORMAT: str = "%Y-%m-%d-%H-%M-%S"
 _EXAMPLE_DT: str = datetime(year=1970, month=1, day=1).strftime(_DT_FORMAT)
@@ -96,6 +96,22 @@ def _get_resource_id(obj: Content) -> str | None:
     return None
 
 
+def _get_resource_name(obj: Content) -> str | None:
+    if isinstance(obj, list):
+        return next((el for el in map(_get_resource_name, obj) if el), None)
+    if isinstance(obj, dict) and obj.get("type") == "resource_id":
+        return _get_field(obj, "resource_name", str)
+    return None
+
+
+def _get_resource(obj: Content) -> str | None:
+    if isinstance(obj, list):
+        return next((el for el in map(_get_resource_id, obj) if el), None)
+    if isinstance(obj, dict) and obj.get("type") == "resource_id":
+        return obj
+    return None
+
+
 def _get_command(obj: Content) -> dict | None:
     if isinstance(obj, list):
         return next((el for el in map(_get_command, obj) if el), None)
@@ -138,6 +154,15 @@ class BaseMessage(Base):
     def resource_id(self) -> str | None:
         return _get_resource_id(self.content)
 
+    @property
+    def resource_name(self) -> str | None:
+        res = _get_resource_name(self.content)
+        return res
+
+    @property
+    def resource(self) -> dict | None:
+        return _get_resource(self.content)
+
     @property
     def command(self) -> dict | None:
         return _get_command(self.content)
@@ -162,7 +187,7 @@ class BaseMessage(Base):
         return _DT_FORMAT
 
     @staticmethod
-    def find_resource_id(msg: "BaseMessage", ext: str | None = None, type: str=None) -> str | None:
+    def find_resource_id(msg: "BaseMessage", ext: str | None = None, type: str = None) -> str | None:
         resource_id = msg.resource_id
         if type and type != msg.type:
             return None
@@ -185,6 +210,9 @@ class AIMessage(BaseMessage):
     def action(self) -> str:
         return (self.extra or {}).get("action", "")
 
+    def with_state(self, state: str) -> "AIMessage":
+        return self.model_copy(update=dict(state=state))
+
 
 class MiscMessage(BaseMessage):
     type: Literal["misc"] = "misc"
@@ -242,25 +270,52 @@ class Chat(Base):
     def rfind_in_messages(self, func: Callable[[ChatMessage], T | None]) -> T | None:
         return find_in_messages(self.messages[::-1], func)
 
+    def get_last_user_message(self) -> HumanMessage | None:
+        messages = self.messages
+        if not messages:
+            return []
+        message = messages[-1]
+        return message if isinstance(message, HumanMessage) else None
+
+    def count_messages(self, func: Callable[[ChatMessage], bool]) -> int:
+        return sum(map(func, self.messages))
+
 
 def make_content(
     text: str | None = None,
     *,
     resource_id: str | None = None,
+    resource: dict | None = None,
     command: dict | None = None,
     widget: Widget | None = None,
     content: Content | None = None,
 ) -> Content:
-    resource_id = (resource_id or None) and {"type": "resource_id", "resource_id": resource_id}
+    if resource and resource_id:
+        raise ValueError("Cannot pass both 'resource' and 'resource_id'")
+
+    if resource_id:
+        resource = {"type": "resource_id", "resource_id": resource_id}
+    elif resource:
+        if not isinstance(resource, dict):
+            raise TypeError("'resource' must be a dict")
+        resource_id = resource.get("resource_id")
+        if not resource_id:
+            raise ValueError("'resource' must contain 'resource_id'")
+        resource_name = resource.get("resource_name")
+        resource = {"type": "resource_id", "resource_id": resource_id}
+        if resource_name:
+            resource["resource_name"] = resource_name
+    else:
+        resource = None
+
     command = (command or None) and {"type": "command", "command": command}
 
     content = content if isinstance(content, list) else [content] if content else []
-    content += list(filter(None, [text, resource_id, command, widget]))
+    content += list(filter(None, [text, resource, command, widget]))
     if len(content) == 0:
         content = ""
     elif len(content) == 1:
         content = content[0]
-
     return content
 
 
@@ -273,13 +328,19 @@ def convert_replica_item_to_message(replica: ReplicaItem) -> ChatMessage:
         widget=replica.widget,
     )
     # legacy: eliminate after migration
-    resource_id = (replica.resource_id or None) and {"type": "resource_id", "resource_id": replica.resource_id}
+    if resource_id := replica.resource_id:
+        resource = {"type": "resource_id", "resource_id": resource_id}
+        resource_name = replica.resource_name
+        if resource_name:
+            resource["resource_name"] = resource_name
+    else:
+        resource = None
     body = replica.body
     command = (replica.command or None) and {"type": "command", "command": replica.command}
     widget = replica.widget
     date_time = replica.date_time
 
-    content = list(filter(None, [body, resource_id, command, widget]))
+    content = list(filter(None, [body, resource, command, widget]))
     if len(content) == 0:
         content = ""
     elif len(content) == 1:
@@ -363,6 +424,7 @@ def convert_message_to_replica_item(message: ChatMessage) -> ReplicaItem | None:
         role=role,
         body=message.text,
         resource_id=message.resource_id,
+        resource_name=message.resource_name,
         command=message.command,
         widget=message.widget,
         date_time=message.date_time,
@@ -376,10 +438,11 @@ def convert_message_to_replica_item(message: ChatMessage) -> ReplicaItem | None:
 
 def convert_chat_to_chat_item(chat: Chat) -> ChatItem:
     # legacy: eliminate after migration
-    return ChatItem(
+    res = ChatItem(
         outer_context=convert_context_to_outer_context(chat.context),
         inner_context=dict(replicas=list(map(convert_message_to_replica_item, chat.messages))),
     )
+    return res
 
 
 def parse_chat_item_as_chat(chat_obj: str | dict | ChatItem) -> Chat:
